@@ -1095,7 +1095,7 @@ pem_CreateObject
 #if 0
     pemToken *token;
 #endif
-    int cipher;
+    int cipher = 0;
     char *ivstring = NULL;
     pemInternalObject *listObj = NULL;
     pemObjectListItem *listItem = NULL;
@@ -1116,7 +1116,6 @@ pem_CreateObject
     /* What slot are we adding the object to? */
     fwSlot = NSSCKFWSession_GetFWSlot(fwSession);
     if ((NSSCKFWSlot *) NULL == fwSlot) {
-        *pError = CKR_ATTRIBUTE_VALUE_INVALID;
         *pError = CKR_GENERAL_ERROR;
         return (NSSCKMDObject *) NULL;
 
@@ -1166,11 +1165,8 @@ pem_CreateObject
     }
 
     if (objClass == CKO_CERTIFICATE) {
-        /* TODO: Fix discrepancy between our usage of the return value as
-         * as an int and the declaration as a SECStatus. Typecasting as a
-         * temporary workaround.
-         */
-        nobjs = (int) ReadDERFromFile(&derlist, filename, PR_TRUE, &cipher, &ivstring, PR_TRUE /* certs only */);
+        nobjs = ReadDERFromFile(&derlist, filename, /* ascii */ PR_TRUE,
+                                &cipher, &ivstring, /* certs only */ PR_TRUE);
         if (nobjs < 1)
             goto loser;
 
@@ -1210,36 +1206,45 @@ pem_CreateObject
                 goto loser;
         }
     } else if (objClass == CKO_PRIVATE_KEY) {
-        /* Brute force: find the id of the certificate, if any, in this slot */
         int i;
         SECItem certDER;
         PRBool added;
 
-        nobjs = ReadDERFromFile(&derlist, filename, PR_TRUE, &cipher, &ivstring, PR_FALSE /* keys only */);
+        nobjs = ReadDERFromFile(&derlist, filename, /* ascii */ PR_TRUE,
+                                &cipher, &ivstring, /* keys only */ PR_FALSE);
         if (nobjs < 1)
             goto loser;
 
         certDER.len = 0; /* in case there is no equivalent cert */
         certDER.data = NULL;
 
+        /* Brute force: find the id of the certificate, if any, in this slot */
         objid = -1;
-        for (i = 0; i < pem_nobjs; i++) {
+        for (i = pem_nobjs - 1; 0 <= i; i--) {
             if (NULL == pem_objs[i])
                 continue;
 
-            if ((slotID == pem_objs[i]->slotID) && (pem_objs[i]->type == pemCert)) {
-                objid = atoi(pem_objs[i]->id.data);
-                certDER.data =
-                    (void *) NSS_ZAlloc(NULL, pem_objs[i]->derCert->len);
+            if (slotID != pem_objs[i]->slotID)
+                continue;
 
-                if (certDER.data == NULL)
-                    goto loser;
+            if (pem_objs[i]->type != pemCert)
+                continue;
 
-                certDER.len = pem_objs[i]->derCert->len;
-                memcpy(certDER.data,
-                        pem_objs[i]->derCert->data,
-                        pem_objs[i]->derCert->len);
-            }
+            if (atoi(pem_objs[i]->id.data) != pem_nobjs)
+                /* not a certificate that refers to the key being added */
+                continue;
+
+            objid = pem_nobjs;
+            certDER.data = NSS_ZAlloc(NULL, pem_objs[i]->derCert->len);
+
+            if (certDER.data == NULL)
+                goto loser;
+
+            certDER.len = pem_objs[i]->derCert->len;
+            memcpy(certDER.data,
+                    pem_objs[i]->derCert->data,
+                    pem_objs[i]->derCert->len);
+            break;
         }
 
         /* We're just adding a key, we'll assume the cert is next */
